@@ -22,7 +22,13 @@ from threading import RLock
 import time
 from serial import SerialException
 from nuvo_serial.configuration import config
-from nuvo_serial.const import ERROR_RESPONSE
+from nuvo_serial.const import (
+    EMIT_LEVEL_ALL,
+    EMIT_LEVEL_EXTERNAL,
+    EMIT_LEVEL_INTERNAL,
+    EMIT_LEVEL_NONE,
+    ERROR_RESPONSE,
+)
 from nuvo_serial.exceptions import (
     MessageClassificationError,
     MessageFormatError,
@@ -34,6 +40,7 @@ from nuvo_serial.message import (
     process_message,
     OKResponse,
     Paging,
+    Party,
     SourceConfiguration,
     ZoneAllOff,
     ZoneButton,
@@ -209,28 +216,67 @@ class SyncRequest:
 
 class MsgBus:
     def __init__(self) -> None:
+        self.emit_level = EMIT_LEVEL_ALL
         self.subscribers: dict[str, set[Callable[..., Coroutine]]] = {}
+        self.internal_subscribers: dict[str, set[Callable[..., Coroutine]]] = {}
 
     def add_subscriber(
-        self, subscriber: Callable[..., Coroutine], event_name: str
+        self,
+        subscriber: Callable[..., Coroutine],
+        event_name: str,
+        internal: bool = False,
     ) -> None:
-        if not self.subscribers.get(event_name, None):
-            self.subscribers[event_name] = {subscriber}
+        if internal:
+            if not self.internal_subscribers.get(event_name, None):
+                self.internal_subscribers[event_name] = {subscriber}
+            else:
+                self.internal_subscribers[event_name].add(subscriber)
         else:
-            self.subscribers[event_name].add(subscriber)
+            if not self.subscribers.get(event_name, None):
+                self.subscribers[event_name] = {subscriber}
+            else:
+                self.subscribers[event_name].add(subscriber)
 
     def remove_subscriber(
-        self, subscriber: Callable[..., Coroutine], event_name: str
+        self,
+        subscriber: Callable[..., Coroutine],
+        event_name: str,
+        internal: bool = False,
     ) -> None:
-        self.subscribers[event_name].remove(subscriber)
-        if len(self.subscribers[event_name]) == 0:
-            del self.subscribers[event_name]
 
-    # def emit_event(self, event_name: NuvoMsgType, event: NuvoClass) -> None:
-    def emit_event(self, event_name: str, event: NuvoClass) -> None:
+        if internal:
+            self.internal_subscribers[event_name].remove(subscriber)
+            if len(self.internal_subscribers[event_name]) == 0:
+                del self.internal_subscribers[event_name]
+        else:
+            self.subscribers[event_name].remove(subscriber)
+
+            if len(self.subscribers[event_name]) == 0:
+                del self.subscribers[event_name]
+
+    def set_emit_level(self, emit_level: str) -> None:
+        self.emit_level = emit_level
+
+    def emit_event(
+            self, event_name: str, event: NuvoClass, emit_level: Optional[str] = None) -> None:
         message = {"event_name": event_name, "event": event}
-        for subscriber in self.subscribers.get(event_name, set()):
-            asyncio.create_task(subscriber(message))
+        if not emit_level:
+            emit_level = self.emit_level
+
+        if emit_level == EMIT_LEVEL_NONE:
+            return
+
+        if (
+            emit_level == EMIT_LEVEL_INTERNAL or emit_level == EMIT_LEVEL_ALL
+        ):
+            for subscriber in self.internal_subscribers.get(event_name, set()):
+                asyncio.create_task(subscriber(message))
+
+        if (
+            emit_level == EMIT_LEVEL_EXTERNAL or emit_level == EMIT_LEVEL_ALL
+        ):
+            for subscriber in self.subscribers.get(event_name, set()):
+                asyncio.create_task(subscriber(message))
 
 
 class AsyncConnection:
@@ -325,9 +371,11 @@ class AsyncConnection:
         ...
 
     @overload
-    async def send_message(
-        self, msg: str, message_types: Literal["Paging"],
-    ) -> Paging:
+    async def send_message(self, msg: str, message_types: Literal["Paging"],) -> Paging:
+        ...
+
+    @overload
+    async def send_message(self, msg: str, message_types: Literal["Party"],) -> Party:
         ...
 
     @overload
@@ -344,7 +392,7 @@ class AsyncConnection:
 
     @overload
     async def send_message(
-        self, msg: str, message_types: Literal["ZoneStatus"]
+            self, msg: str, message_types: Literal["ZoneStatus"], emit_level: Optional[str] = EMIT_LEVEL_ALL
     ) -> ZoneStatus:
         ...
 
@@ -375,7 +423,8 @@ class AsyncConnection:
         # self, msg: str, message_types: Union[NuvoMsgType, Tuple[NuvoMsgType, ...]]
 
     async def send_message(
-        self, msg: str, message_types: Union[NuvoMsgType, Tuple[NuvoMsgType, ...]]
+        self, msg: str, message_types: Union[NuvoMsgType, Tuple[NuvoMsgType, ...]],
+        emit_level: Optional[str] = EMIT_LEVEL_ALL
     ) -> NuvoClass:
         """Send a message to the Nuvo and wait for a response."""
 
@@ -447,7 +496,7 @@ class AsyncConnection:
             )
         else:
             self._start_streaming_reader()
-            self._bus.emit_event(response[0], response[1])
+            self._bus.emit_event(response[0], response[1], emit_level)
             return response[1]
 
     async def _send(self, message: bytes) -> None:
@@ -609,7 +658,9 @@ class AsyncConnection:
                 # There message has been classified but it may not be the wanted
                 # message_type
                 if processed_type == ERROR_RESPONSE:
-                    err_msg = "Message produced an error response from the Nuvo controller"
+                    err_msg = (
+                        "Message produced an error response from the Nuvo controller"
+                    )
                     _LOGGER.debug("RESPONSEREADER: MessageResponseError: %s", err_msg)
                     raise MessageResponseError(err_msg)
                 if processed_type in message_types:
