@@ -8,6 +8,7 @@ OK_RESPONSE: Final = "OKResponse"
 SOURCE_CONFIGURATION: Final = "SourceConfiguration"
 SYSTEM_VERSION: Final = "Version"
 SYSTEM_PAGING: Final = "Paging"
+SYSTEM_PARTY: Final = "Party"
 ZONE_STATUS: Final = "ZoneStatus"
 ZONE_CONFIGURATION: Final = "ZoneConfiguration"
 ZONE_VOLUME_CONFIGURATION: Final = "ZoneVolumeConfiguration"
@@ -28,3 +29,8 @@ ZONE_BUTTON_TRANSFORM = {
 
 RESPONSE_STRING_OK = "#OK"
 RESPONSE_STRING_ERROR = "#?"
+
+EMIT_LEVEL_ALL = "emit_all"
+EMIT_LEVEL_EXTERNAL = "emit_external"
+EMIT_LEVEL_INTERNAL = "emit_internal"
+EMIT_LEVEL_NONE = "emit_none"
